void ReloadStyle() {
    ColorText = GetColor("Text", ColorText);
    ColorLoadTextHighlight = GetColor("LoadTextHighlight", ColorLoadTextHighlight);
    ColorBackground = GetColor("Background", ColorBackground);
    ColorInactiveWindow = GetColor("InactiveWindow", ColorInactiveWindow);
    ColorTextLineNumbers = GetColor("TextLineNumbers", ColorTextLineNumbers);
    ColorLineHighlight = GetColor("LineHighlight", ColorLineHighlight);
    ColorMainCaret = GetColor("MainCaret", ColorMainCaret);
    ColorSubCaret = GetColor("SubCaret", ColorSubCaret);
    ColorSelection = GetColor("Selection", ColorSelection);
    ColorWhitespaceDuringSelection = GetColor("WhitespaceDuringSelection", ColorWhitespaceDuringSelection);
    ColorMouseUnderline = GetColor("MouseUnderline", ColorMouseUnderline);
    ColorCaretUnderline = GetColor("CaretUnderline", ColorCaretUnderline);
    ColorFuzzySearchLineHighlight = GetColor("FuzzySearchLineHighlight", ColorFuzzySearchLineHighlight);
    ColorScrollbarBackground = GetColor("ScrollbarBackground", ColorScrollbarBackground);
    ColorScrollbarScroller = GetColor("ScrollbarScroller", ColorScrollbarScroller);
    ColorScrollbarScrollerSelected = GetColor("ScrollbarScrollerSelected", ColorScrollbarScrollerSelected);
    ColorTitleBarText = GetColor("TitleBarText", ColorTitleBarText);
    ColorTitleBarBackground = GetColor("TitleBarBackground", ColorTitleBarBackground);
    ColorTitleBarActiveBackground = GetColor("TitleBarActiveBackground", ColorTitleBarActiveBackground);
    ColorTitleBarSelection = GetColor("TitleBarSelection", ColorTitleBarSelection);
    ColorResizerBackground = GetColor("ResizerBackground", ColorResizerBackground);
    ColorResizerOutline = GetColor("ResizerOutline", ColorResizerOutline);
    StyleWaitForEvents = GetStyleInt("WaitForEvents", StyleWaitForEvents);
    StyleDrawLineNumbers = GetStyleInt("DrawLineNumbers", StyleDrawLineNumbers);
    StyleDrawScrollbar = GetStyleInt("DrawScrollbar", StyleDrawScrollbar);
    StyleIndentSize = GetStyleInt("IndentSize", StyleIndentSize);
    StyleFontSize = GetStyleInt("FontSize", StyleFontSize);
    StyleFontFilter = GetStyleInt("FontFilter", StyleFontFilter);
    StyleFont = GetStyleString("Font", StyleFont);
    StyleVCVarsall = GetStyleString("VCVarsall", StyleVCVarsall);
    StyleUndoMergeTimeout = GetStyleFloat("UndoMergeTimeout", StyleUndoMergeTimeout);
}